"""This contains emperically derived constants from Hutto and Gilbert (2014)"""

# (empirically derived mean sentiment intensity rating increase for booster words)
B_INCR = 0.293
B_DECR = -0.293

# (empirically derived mean sentiment intensity rating increase for using ALLCAPs to emphasize a word)
C_INCR = 0.733  # capitatilization scaler
N_SCALAR = -0.74


BEFORE_BUT_SCALAR = 0.5
AFTER_BUT_SCALAR = 1.5
